'use strict';

var _require = require('./emitter'),
    createChangesFunnelEmitter = _require.createChangesFunnelEmitter;

var createTransactions = function createTransactions(models) {
  return models.map(function (model) {
    return model.createTransaction();
  });
};
var commitTransactions = function commitTransactions(models) {
  return models.map(function (model) {
    return model.commitTransaction();
  });
};
var abortTransactions = function abortTransactions(models) {
  return models.map(function (model) {
    return model.abortTransaction();
  });
};
var emitChanges = function emitChanges(models) {
  return createChangesFunnelEmitter({ models: models }).emitChange();
};
var normalized = function normalized(value) {
  return Array.isArray(value) && value || [value];
};

var createNormalizedAction = function createNormalizedAction(handler, models) {
  return function () {
    var res = null;
    var catchedError = null;

    createTransactions(models);

    try {
      res = handler.apply(undefined, arguments);
      commitTransactions(models);
    } catch (error) {
      catchedError = error;
      abortTransactions(models);
      throw error;
    }

    if (!catchedError) {
      emitChanges(models);
    }

    return res;
  };
};

var createAction = function createAction(handler, models) {
  return createNormalizedAction(handler, normalized(models));
};
var runAsAction = function runAsAction(handler, models) {
  return createAction(handler, models)();
};

module.exports = {
  createAction: createAction,
  runAsAction: runAsAction
};
